@@ -2,13 +2,13 @@ package Math::Calc::Parser;
 use strict;
 use warnings;
 use Carp 'croak';
+use Exporter ();
 use List::Util 'reduce';
 use Math::Complex;
 use POSIX qw/ceil floor/;
 use Scalar::Util qw/blessed looks_like_number/;
-require Exporter;
 
-our $VERSION = '1.000';
+our $VERSION = '1.001';
 our @ISA = 'Exporter';
 our @EXPORT_OK = 'calc';
 our $ERROR;
@@ -46,7 +46,7 @@ use constant ROUND_HALF => 0.50000000000008;
 	my (%lower_prec, %higher_prec);
 	$higher_prec{$_} = 1 for keys %operators;
 	foreach my $set (@op_precedence) {
-		delete $higher_prec{$_} for @$set;
+		delete @higher_prec{@$set};
 		foreach my $op (@$set) {
 			$operators{$op}{equal_to}{$_} = 1 for @$set;
 			$operators{$op}{lower_than}{$_} = 1 for keys %higher_prec;
@@ -55,13 +55,10 @@ use constant ROUND_HALF => 0.50000000000008;
 		$lower_prec{$_} = 1 for @$set;
 	}
 	
-	sub _operator {
-		my $oper = shift;
-		croak 'No operator passed' unless defined $oper;
-		return undef unless exists $operators{$oper};
-		return $operators{$oper};
-	}
-	
+	sub _operator { $operators{shift()} }
+}
+
+{
 	sub _real { blessed $_[0] ? $_[0]->Re : $_[0] }
 	sub _each { blessed $_[0] ? cplx($_[1]->($_[0]->Re), $_[1]->($_[0]->Im)) : $_[1]->($_[0]) }
 	
@@ -77,8 +74,8 @@ use constant ROUND_HALF => 0.50000000000008;
 		'/'   => { args => 2, code => sub { $_[0] / $_[1] } },
 		'%'   => { args => 2, code => sub { _real($_[0]) % _real($_[1]) } },
 		'^'   => { args => 2, code => sub { $_[0] ** $_[1] } },
-		'!'   => { args => 1, code => sub { die "Factorial of negative number" if _real($_[0]) < 0;
-		                                    die "Factorial of infinity" if _real($_[0]) == 'inf';
+		'!'   => { args => 1, code => sub { die 'Factorial of negative number' if _real($_[0]) < 0;
+		                                    die 'Factorial of infinity' if _real($_[0]) == 'inf';
 		                                    reduce { $a * $b } 1, 1.._real($_[0]) } },
 		'u-'  => { args => 1, code => sub { -$_[0] } },
 		'u+'  => { args => 1, code => sub { +$_[0] } },
@@ -104,57 +101,34 @@ use constant ROUND_HALF => 0.50000000000008;
 	);
 	
 	sub _default_functions { +{%functions} }
-	
+}
+
+{
 	my $singleton;
-	sub _instance {
-		return $_[0] if blessed $_[0];
-		$singleton = $_[0]->new unless defined $singleton;
-		return $singleton;
-	}
+	sub _instance { blessed $_[0] ? $_[0] : ($singleton ||= $_[0]->new) }
 }
 
-sub new {
-	my $class = shift;
-	my $self = {};
-	bless $self, $class;
-	return $self;
-}
+sub calc ($) { _instance(__PACKAGE__)->evaluate($_[0]) }
 
-sub _functions {
-	my $self = shift;
-	$self->{_functions} = _default_functions() unless defined $self->{_functions};
-	return $self->{_functions};
-}
+sub new { bless {}, shift }
 
-sub error {
-	my $self = _instance(shift);
-	return exists $self->{error} ? $self->{error} : undef;
-}
+sub error { _instance(shift)->{error} }
 
-sub _set_error {
-	my ($self, $error) = @_;
-	$self->{error} = $error;
-	return $self;
-}
-
-sub clear_error {
-	my $self = shift;
-	delete $self->{error};
-}
+sub _functions { shift->{_functions} ||= _default_functions() }
 
 sub add_functions {
 	my ($self, %functions) = @_;
 	foreach my $name (keys %functions) {
-		croak "Function \"$name\" has invalid name" unless $name =~ m/\A[a-z]\w*\z/i;
+		croak qq{Function "$name" has invalid name} unless $name =~ m/\A[a-z]\w*\z/i;
 		my $definition = $functions{$name};
 		$definition = { args => 0, code => $definition } if ref $definition eq 'CODE';
-		croak "No argument count for function \"$name\""
+		croak qq{No argument count for function "$name"}
 			unless defined (my $args = $definition->{args});
-		croak "Invalid argument count for function \"$name\""
+		croak qq{Invalid argument count for function "$name"}
 			unless $args =~ m/\A\d+\z/ and $args >= 0;
-		croak "No coderef for function \"$name\""
+		croak qq{No coderef for function "$name"}
 			unless defined (my $code = $definition->{code});
-		croak "Invalid coderef for function \"$name\"" unless ref $code eq 'CODE';
+		croak qq{Invalid coderef for function "$name"} unless ref $code eq 'CODE';
 		$self->_functions->{$name} = { args => $args, code => $code };
 	}
 	return $self;
@@ -162,8 +136,8 @@ sub add_functions {
 
 sub remove_functions {
 	my ($self, @functions) = @_;
-	foreach my $name (grep { defined } @functions) {
-		next unless exists $self->_functions->{$name};
+	foreach my $name (@functions) {
+		next unless defined $name;
 		next if defined _operator($name); # Do not remove operator functions
 		delete $self->_functions->{$name};
 	}
@@ -217,7 +191,7 @@ sub parse {
 			_shunt_number(\@expr_queue, \@oper_stack, $token);
 			$binop_possible = 1;
 		} elsif ($token =~ m/\A\w+\z/) {
-			die "Invalid function \"$token\"\n" unless exists $self->_functions->{$token};
+			die qq{Invalid function "$token"\n} unless exists $self->_functions->{$token};
 			if ($self->_functions->{$token}{args} > 0) {
 				_shunt_function_with_args(\@expr_queue, \@oper_stack, $token);
 				$binop_possible = 0;
@@ -226,7 +200,7 @@ sub parse {
 				$binop_possible = 1;
 			}
 		} else {
-			die "Unknown token \"$token\"\n";
+			die qq{Unknown token "$token"\n};
 		}
 	}
 	
@@ -303,8 +277,6 @@ sub _shunt_comma {
 	return 1;
 }
 
-sub calc ($) { __PACKAGE__->evaluate($_[0]) }
-
 sub evaluate {
 	my ($self, $expr) = @_;
 	$self = _instance($self);
@@ -320,17 +292,21 @@ sub evaluate {
 			my $num_args = $function->{args};
 			die "Malformed expression\n" if @eval_stack < $num_args;
 			my @args = $num_args > 0 ? splice @eval_stack, -$num_args : ();
-			my ($result, $error);
+			my ($result, $errored, $error);
 			{
 				local $@;
-				eval { $result = $function->{code}(@args); 1 } or $error = $@;
+				unless (eval { $result = $function->{code}(@args); 1 }) {
+					$errored = 1;
+					$error = $@;
+				}
 			}
-			if (defined $error) {
+			if ($errored) {
+				$error = '' unless defined $error;
 				$error =~ s/ at .+? line \d+\.$//i;
 				chomp $error;
-				die "$error\n";
+				die qq{Error in function "$token": $error\n};
 			}
-			die "Undefined result from function or operator \"$token\"\n" unless defined $result;
+			die qq{Undefined result from function "$token"\n} unless defined $result;
 			{
 				no warnings 'numeric';
 				push @eval_stack, 0+$result;
@@ -338,7 +314,7 @@ sub evaluate {
 		} elsif (looks_like_number $token) {
 			push @eval_stack, $token;
 		} else {
-			die "Invalid function or operator \"$token\"\n";
+			die qq{Invalid function "$token"\n};
 		}
 	}
 	
@@ -350,14 +326,13 @@ sub evaluate {
 sub try_evaluate {
 	my ($self, $expr) = @_;
 	$self = _instance($self);
-	$self->clear_error;
+	delete $self->{error};
 	undef $ERROR;
 	local $@;
 	my $result;
 	unless (eval { $result = $self->evaluate($expr); 1 }) {
-		my $error = $@;
-		chomp $error;
-		$self->_set_error($ERROR = $error);
+		chomp(my $error = $@);
+		$self->{error} = $ERROR = $error;
 		return undef;
 	}
 	return $result;
@@ -431,35 +406,37 @@ customized using L</"add_functions"> and L</"remove_functions">.
 Compact exportable function wrapping L</"evaluate"> for string expressions.
 Throws an exception on error. See L<ath> for easy compact one-liners.
 
-=head1 ATTRIBUTES
-
-=head2 error
-
-  my $result = $parser->try_evaluate('2//');
-  die $parser->error unless defined $result;
-
-Returns the error message after a failed L</"try_evaluate">.
-
 =head1 METHODS
 
+Aside from C<add_functions> and C<remove_functions>, all methods can be called
+as class methods, and will act on a singleton object with the default functions
+available.
+
+=head2 new
+
+  my $parser = Math::Calc::Parser->new;
+
+Creates a new L<Math::Calc::Parser> object.
+
 =head2 parse
 
   my $parsed = Math::Calc::Parser->parse('5 / e^(i*pi)');
+  my $parsed = $parser->parse('3pi');
 
-Parses a mathematical expression. Can be called as either an object or class
-method. On success, returns an array reference representation of the expression
-in RPN notation which can be passed to L</"evaluate">. Throws an exception on
-failure.
+Parses a mathematical expression. On success, returns an array reference
+representation of the expression in RPN notation which can be passed to
+L</"evaluate">. Throws an exception on failure.
 
 =head2 evaluate
 
   my $result = Math::Calc::Parser->evaluate($parsed);
   my $result = Math::Calc::Parser->evaluate('log rand 7');
+  my $result = $parser->evaluate('round 13/3');
 
-Evaluates a mathematical expression. Can be called as either an object or class
-method, and the argument can be either an arrayref from L</"parse"> or a string
-expression. Returns the result of the expression on success or throws an
-exception on failure.
+Evaluates a mathematical expression. The argument can be either an arrayref
+from L</"parse"> or a string expression which will be passed to L</"parse">.
+Returns the result of the expression on success or throws an exception on
+failure.
 
 =head2 try_evaluate
 
@@ -478,7 +455,16 @@ exception on failure.
 Same as L</"evaluate"> but instead of throwing an exception on failure, returns
 undef and sets the L</"error"> attribute to the error message. The error
 message for the most recent L</"try_evaluate"> call can also be retrieved from
-the global variable C<$Math::Calc::Parser::ERROR>.
+the package variable C<$Math::Calc::Parser::ERROR>.
+
+=head2 error
+
+  my $result = Math::Calc::Parser->try_evaluate('(i');
+  die Math::Calc::Parser->error unless defined $result;
+  my $result = $parser->try_evaluate('2//');
+  die $parser->error unless defined $result;
+
+Returns the error message after a failed L</"try_evaluate">.
 
 =head2 add_functions
 
@@ -508,12 +494,12 @@ L</"add_functions">.
 =head1 OPERATORS
 
 L<Math::Calc::Parser> recognizes the following operators with their usual
-definitions.
+mathematical definitions.
 
   +, -, *, /, %, ^, !, <<, >>
 
-Note: C<+> and C<-> can represent a unary operation (negation) in addition to
-addition and subtraction.
+Note: C<+> and C<-> can represent both binary addition/subtraction and unary
+negation.
 
 =head1 DEFAULT FUNCTIONS
 
